@@ -25,48 +25,23 @@ const CALLEE_SAVED_REGS: &[&str] = &["rbx", "rbp", "r12", "r13", "r14", "r15"];
 
 async fn get_caller_by_cfi<P>(
     ctx: &CONTEXT_AMD64,
-    callee: &StackFrame,
-    grand_callee: Option<&StackFrame>,
-    stack_memory: UnifiedMemory<'_, '_>,
-    modules: &MinidumpModuleList,
-    symbol_provider: &P,
+    args: &GetCallerFrameArgs<'_, P>,
 ) -> Option<StackFrame>
 where
     P: SymbolProvider + Sync,
 {
     trace!("trying cfi");
 
-    let valid = &callee.context.valid;
-    if let MinidumpContextValidity::Some(ref which) = valid {
+    if let MinidumpContextValidity::Some(ref which) = args.valid() {
         if !which.contains(STACK_POINTER_REGISTER) {
             return None;
         }
     }
 
-    let module = modules.module_at_address(callee.instruction)?;
+    let mut stack_walker = CfiStackWalker::from_ctx_and_args(ctx, args, callee_forwarded_regs)?;
 
-    let grand_callee_parameter_size = grand_callee.and_then(|f| f.parameter_size).unwrap_or(0);
-    let has_grand_callee = grand_callee.is_some();
-
-    let mut stack_walker = CfiStackWalker {
-        instruction: callee.instruction,
-        has_grand_callee,
-        grand_callee_parameter_size,
-
-        callee_ctx: ctx,
-        callee_validity: valid,
-
-        // Default to forwarding all callee-saved regs verbatim.
-        // The CFI evaluator may clear or overwrite these values.
-        // The stack pointer and instruction pointer are not included.
-        caller_ctx: ctx.clone(),
-        caller_validity: callee_forwarded_regs(valid),
-
-        stack_memory,
-    };
-
-    symbol_provider
-        .walk_frame(module, &mut stack_walker)
+    args.symbol_provider
+        .walk_frame(stack_walker.module, &mut stack_walker)
         .await?;
     let caller_ip = stack_walker.caller_ctx.rip;
     let caller_sp = stack_walker.caller_ctx.rsp;
@@ -104,29 +79,15 @@ fn callee_forwarded_regs(valid: &MinidumpContextValidity) -> HashSet<&'static st
 
 fn get_caller_by_frame_pointer<P>(
     ctx: &CONTEXT_AMD64,
-    callee: &StackFrame,
-    stack_memory: UnifiedMemory<'_, '_>,
-    _modules: &MinidumpModuleList,
-    system_info: &SystemInfo,
-    _symbol_provider: &P,
+    args: &GetCallerFrameArgs<'_, P>,
 ) -> Option<StackFrame>
 where
     P: SymbolProvider + Sync,
 {
-    // On Windows x64, frame-pointer unwinding purely with the data on the stack
-    // is not possible, as proper unwinding requires access to `UNWIND_INFO`,
-    // because the frame pointer does not necessarily point to the end of the
-    // frame.
-    // In particular, the docs state that:
-    // > [The frame register] offset permits pointing the FP register into the
-    // > middle of the local stack allocation [...]
-    // https://docs.microsoft.com/en-us/cpp/build/exception-handling-x64
-    if system_info.os == Os::Windows {
-        return None;
-    }
+    let stack_memory = args.stack_memory;
 
     trace!("trying frame pointer");
-    if let MinidumpContextValidity::Some(ref which) = callee.context.valid {
+    if let MinidumpContextValidity::Some(ref which) = args.valid() {
         if !which.contains(FRAME_POINTER_REGISTER) {
             return None;
         }
@@ -166,32 +127,66 @@ where
         // drowning the rest of the code in checked_add.
         return None;
     }
-    let caller_ip = stack_memory.get_memory_at_address(last_bp + POINTER_WIDTH)?;
-    let caller_bp = stack_memory.get_memory_at_address(last_bp)?;
-    let caller_sp = last_bp + POINTER_WIDTH * 2;
 
-    // If the recovered ip is not a canonical address it can't be
-    // the return address, so bp must not have been a frame pointer.
+    let resolve = |offset_max_scan, offset_step| -> Option<(u64, u64, u64)> {
+        for offset in 0..=offset_max_scan {
+            let offset = offset * offset_step;
+            let caller_ip = stack_memory.get_memory_at_address(last_bp + offset + POINTER_WIDTH)?;
+            let caller_bp = stack_memory.get_memory_at_address(last_bp + offset)?;
+            let caller_sp = last_bp + offset + POINTER_WIDTH * 2;
 
-    // Since we're assuming coherent frame pointers, check that the frame pointers
-    // and stack pointers are well-ordered.
-    if caller_sp <= last_bp || caller_bp < caller_sp {
-        trace!("rejecting frame pointer result for unreasonable frame pointer");
-        return None;
-    }
-    // Since we're assuming coherent frame pointers, check that the resulting
-    // frame pointer is still inside stack memory.
-    let _unused: Pointer = stack_memory.get_memory_at_address(caller_bp)?;
-    // Don't accept obviously wrong instruction pointers.
-    if is_non_canonical(caller_ip) {
-        trace!("rejecting frame pointer result for unreasonable instruction pointer");
-        return None;
-    }
-    // Don't accept obviously wrong stack pointers.
-    if !stack_seems_valid(caller_sp, last_sp, stack_memory) {
-        trace!("rejecting frame pointer result for unreasonable stack pointer");
-        return None;
-    }
+            // If the recovered ip is not a canonical address it can't be
+            // the return address, so bp must not have been a frame pointer.
+
+            // Since we're assuming coherent frame pointers, check that the frame pointers
+            // and stack pointers are well-ordered.
+            if caller_sp <= last_bp || caller_bp < caller_sp {
+                trace!("rejecting frame pointer result for unreasonable frame pointer");
+                continue;
+            }
+
+            // Since we're assuming coherent frame pointers, check that the resulting
+            // frame pointer is still inside stack memory.
+            let _unused: Pointer = stack_memory.get_memory_at_address(caller_bp)?;
+            // Don't accept obviously wrong instruction pointers.
+            if is_non_canonical(caller_ip) {
+                trace!("rejecting frame pointer result for unreasonable instruction pointer");
+                continue;
+            }
+            // Don't accept obviously wrong stack pointers.
+            if !stack_seems_valid(caller_sp, last_sp, stack_memory) {
+                trace!("rejecting frame pointer result for unreasonable stack pointer");
+                continue;
+            }
+
+            return Some((caller_ip, caller_bp, caller_sp));
+        }
+        None
+    };
+
+    let (caller_ip, caller_bp, caller_sp) = match args.system_info.os {
+        // On Windows x64, frame-pointer unwinding purely with the data on the stack
+        // is not possible, as proper unwinding requires access to `UNWIND_INFO`,
+        // because the frame pointer does not necessarily point to the end of the
+        // frame.
+        //
+        // In particular, the docs state that:
+        // > [The frame register] offset permits pointing the FP register into the
+        // > middle of the local stack allocation [...]
+        // https://docs.microsoft.com/en-us/cpp/build/exception-handling-x64
+        //
+        // But we can combine traditional frame-pointer unwinding with a educated
+        // stack scan.
+        // We know the given frame pointer points to a maximum 240 bytes into the middle
+        // of the stack. We also know the offset steps in 16 byte increments.
+        // Using this information we can scan up the stack to find the caller instruction
+        // address as well as the adjacent caller frame pointer.
+        //
+        // If this educated scan ends up failing, there is still the fallback to traditional
+        // stack scanning to find the next frame.
+        Os::Windows => resolve(15, 2 * POINTER_WIDTH)?,
+        _ => resolve(0, 0)?,
+    };
 
     trace!(
         "frame pointer seems valid -- caller_ip: 0x{:016x}, caller_sp: 0x{:016x}",
@@ -218,22 +213,21 @@ where
 
 async fn get_caller_by_scan<P>(
     ctx: &CONTEXT_AMD64,
-    callee: &StackFrame,
-    stack_memory: UnifiedMemory<'_, '_>,
-    modules: &MinidumpModuleList,
-    symbol_provider: &P,
+    args: &GetCallerFrameArgs<'_, P>,
 ) -> Option<StackFrame>
 where
     P: SymbolProvider + Sync,
 {
     trace!("trying scan");
+    let stack_memory = args.stack_memory;
+
     // Stack scanning is just walking from the end of the frame until we encounter
     // a value on the stack that looks like a pointer into some code (it's an address
     // in a range covered by one of our modules). If we find such an instruction,
     // we assume it's an ip value that was pushed by the CALL instruction that created
     // the current frame. The next frame is then assumed to end just before that
     // ip value.
-    let last_bp = match callee.context.valid {
+    let last_bp = match args.valid() {
         MinidumpContextValidity::All => Some(ctx.rbp),
         MinidumpContextValidity::Some(ref which) => {
             if !which.contains(STACK_POINTER_REGISTER) {
@@ -255,7 +249,7 @@ where
 
     // Breakpad devs found that the first frame of an unwind can be really messed up,
     // and therefore benefits from a longer scan. Let's do it too.
-    let scan_range = if let FrameTrust::Context = callee.trust {
+    let scan_range = if let FrameTrust::Context = args.callee_frame.trust {
         extended_scan_range
     } else {
         default_scan_range
@@ -264,7 +258,7 @@ where
     for i in 0..scan_range {
         let address_of_ip = last_sp.checked_add(i * POINTER_WIDTH)?;
         let caller_ip = stack_memory.get_memory_at_address(address_of_ip)?;
-        if instruction_seems_valid(caller_ip, modules, symbol_provider).await {
+        if instruction_seems_valid(caller_ip, args.modules, args.symbol_provider).await {
             // ip is pushed by CALL, so sp is just address_of_ip + ptr
             let caller_sp = address_of_ip.checked_add(POINTER_WIDTH)?;
 
@@ -417,63 +411,53 @@ fn is_non_canonical(ptr: Pointer) -> bool {
     ptr > 0x7FFFFFFFFFFF && ptr < 0xFFFF800000000000
 }
 
-#[async_trait::async_trait]
-impl Unwind for CONTEXT_AMD64 {
-    async fn get_caller_frame<P>(
-        &self,
-        callee: &StackFrame,
-        grand_callee: Option<&StackFrame>,
-        stack_memory: Option<UnifiedMemory<'_, '_>>,
-        modules: &MinidumpModuleList,
-        system_info: &SystemInfo,
-        syms: &P,
-    ) -> Option<StackFrame>
-    where
-        P: SymbolProvider + Sync,
-    {
-        let stack = stack_memory?;
-
-        // .await doesn't like closures, so don't use Option chaining
-        let mut frame = None;
-        if frame.is_none() {
-            frame = get_caller_by_cfi(self, callee, grand_callee, stack, modules, syms).await;
-        }
-        if frame.is_none() {
-            frame = get_caller_by_frame_pointer(self, callee, stack, modules, system_info, syms);
-        }
-        if frame.is_none() {
-            frame = get_caller_by_scan(self, callee, stack, modules, syms).await;
-        }
-        let mut frame = frame?;
-
-        // We now check the frame to see if it looks like unwinding is complete,
-        // based on the frame we computed having a nonsense value. Returning
-        // None signals to the unwinder to stop unwinding.
-
-        // if the instruction is within the first ~page of memory, it's basically
-        // null, and we can assume unwinding is complete.
-        if frame.context.get_instruction_pointer() < 4096 {
-            trace!("instruction pointer was nullish, assuming unwind complete");
-            return None;
-        }
-        // If the new stack pointer is at a lower address than the old,
-        // then that's clearly incorrect. Treat this as end-of-stack to
-        // enforce progress and avoid infinite loops.
-        if frame.context.get_stack_pointer() <= self.rsp {
-            trace!("stack pointer went backwards, assuming unwind complete");
-            return None;
-        }
-
-        // Ok, the frame now seems well and truly valid, do final cleanup.
-
-        // A caller's ip is the return address, which is the instruction
-        // *after* the CALL that caused us to arrive at the callee. Set
-        // the value to one less than that, so it points within the
-        // CALL instruction. This is important because we use this value
-        // to lookup the CFI we need to unwind the next frame.
-        let ip = frame.context.get_instruction_pointer();
-        frame.instruction = ip - 1;
-
-        Some(frame)
+pub async fn get_caller_frame<P>(
+    ctx: &CONTEXT_AMD64,
+    args: &GetCallerFrameArgs<'_, P>,
+) -> Option<StackFrame>
+where
+    P: SymbolProvider + Sync,
+{
+    // .await doesn't like closures, so don't use Option chaining
+    let mut frame = None;
+    if frame.is_none() {
+        frame = get_caller_by_cfi(ctx, args).await;
     }
+    if frame.is_none() {
+        frame = get_caller_by_frame_pointer(ctx, args);
+    }
+    if frame.is_none() {
+        frame = get_caller_by_scan(ctx, args).await;
+    }
+    let mut frame = frame?;
+
+    // We now check the frame to see if it looks like unwinding is complete,
+    // based on the frame we computed having a nonsense value. Returning
+    // None signals to the unwinder to stop unwinding.
+
+    // if the instruction is within the first ~page of memory, it's basically
+    // null, and we can assume unwinding is complete.
+    if frame.context.get_instruction_pointer() < 4096 {
+        trace!("instruction pointer was nullish, assuming unwind complete");
+        return None;
+    }
+    // If the new stack pointer is at a lower address than the old,
+    // then that's clearly incorrect. Treat this as end-of-stack to
+    // enforce progress and avoid infinite loops.
+    if frame.context.get_stack_pointer() <= ctx.rsp {
+        trace!("stack pointer went backwards, assuming unwind complete");
+        return None;
+    }
+
+    // Ok, the frame now seems well and truly valid, do final cleanup.
+
+    // A caller's ip is the return address, which is the instruction
+    // *after* the CALL that caused us to arrive at the callee. Set
+    // the value to one less than that, so it points within the
+    // CALL instruction. This is important because we use this value
+    // to lookup the CFI we need to unwind the next frame.
+    let ip = frame.context.get_instruction_pointer();
+    frame.instruction = ip - 1;
+
+    Some(frame)
 }
